//////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      XPLOGON.CPP
//
//  Description
//      This file implements the IXPLogon interface with the methods specified
//      in the MAPI SPI 1.0.
//
//  Author
//      Irving De la Cruz
//
//  Revision: 1.0
//
// Written for Microsoft Windows Developer Support
// Copyright (c) 1994-1995 Microsoft Corporation. All rights reserved.
//
//
#include "XPCUST.H"

#define DO_INFO_TRACES
#ifdef DO_INFO_TRACES
#define InfoTrace(a)   TraceInfoMessage(a)
#else
#define InfoTrace(a)   
#endif // DO_INFO_TRACES

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::CXPLogon()
//
//    Parameters
//      pXPProviderObj  CXPProvider object to which this Logon object
//                      will be attached
//      pSupObj         Pointer to an IMAPISupport object
//      pProps          Array of XP logon properties
//
//    Purpose
//      Constructor of the object. Parameters are passed to initialize the
//      data members with the appropiate values.
//
//    Return Value
//      None
//
CXPLogon::CXPLogon (CXPProvider *   pXPProviderObj,
                    LPMAPISUP       pSupObj,
                    LPSPropValue    pProps,
                    HANDLE          hUIMutex)
{
    InfoTrace ("CXPLogon: Constructor called");
    m_cRef              = 1;
    m_pXPObj            = pXPProviderObj;
    m_pProps            = pProps;
    m_hInstance         = m_pXPObj->m_hInstance;
    m_pIdentityProps    = NULL;
    m_pNextSession      = NULL;
    m_pStatusObj        = NULL;
    m_pszAdrType        = NULL;
    m_hSearch           = INVALID_HANDLE_VALUE;
    m_fPendingProcess   = FALSE;
    m_dwNextID          = 0;
    m_pOptData          = NULL;
    m_cOptData          = 0;
    m_hUIMutex          = hUIMutex;
    m_pSupObj           = pSupObj;
    m_pSupObj->AddRef();
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::~CXPLogon()
//
//    Parameters
//      None
//
//    Purpose
//      Destructor of CXPLogon. Releases memory allocated for internal 
//      properties during the life of this transport logon object.
//
//    Return Value
//      None
//
CXPLogon::~CXPLogon ()
{
    InfoTrace ("CXPLogon: Destructor called");
    gpfnFreeBuffer (m_pIdentityProps);
    m_pIdentityProps = NULL;
    gpfnFreeBuffer (m_pProps);
    m_pProps = NULL;
    gpfnFreeBuffer (m_pOptData);
    m_pOptData = NULL;
    // Delete the memory for the address types string
    delete [] m_pszAdrType;
    m_pszAdrType = NULL;
    // Delete the CMAPIStatus Object, if we have one
    if (m_pStatusObj)
    {
        m_pStatusObj->Release();
        m_pStatusObj = NULL;
    }
    // Release the IMAPISupport object
    m_pSupObj->Release();
    m_pSupObj = NULL;
    CloseHandle (m_hUIMutex);
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::QueryInterface()
//
//    Parameters
//      { Refer to OLE Documentation on this method }
//
//    Purpose
//      Returns a pointer to a interface requested if the interface is 
//      supported and implemented by this object. If it is not supported, it
//      returns NULL
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CXPLogon::QueryInterface (REFIID riid, LPVOID * ppvObj)
{
    // OLE requires NULLing parameter
    *ppvObj = NULL;
    // If this is one of the two IID return an interface pointer to it
    if (riid == IID_IXPLogon || riid == IID_IUnknown)
    {
        *ppvObj = (LPVOID)this;
        // Increase usage count of this object
        AddRef();
        return S_OK;
    }
    // This object does not support the interface requested
    return E_NOINTERFACE;
}

///////////////////////////////////////////////////////////////////////////////
// IXPLogon virtual member functions implementation
//

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::AddressType()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Called by the MAPI Spooler when initializing this XP logon object to 
//      allow the transport to register the address it will handle
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CXPLogon::AddressTypes (ULONG *        pulFlags,
                                     ULONG *        pcAdrType,
                                     LPTSTR **      pppAdrTypeArray,
                                     ULONG *        pcMAPIUID,
                                     LPMAPIUID **   pppMAPIUIDArray)
{
    InfoTrace ("CXPLogon::AddressTypes method called");
    CheckParameters (IXPLogon_AddressTypes, pulFlags);

    // Flags: MAPI_UNICODE for MAPI 1.0
    *pulFlags = fMapiUnicode;

    // The UID for routing message has to be set here. If it is not used,
    // set it NULL
    *pcMAPIUID = 0;
    *pppMAPIUIDArray = NULL;

    // Copy the ADDRESS TYPE string and set the number of addresses this transport handles
    *pcAdrType = 1;
    
    // Copy addresses strings
    LPTSTR pStr = m_pProps[ADDRESS_TYPE].Value.LPSZ;
    HRESULT hResult = S_OK;
    // Allocate memory for the address type string. This memory can be allocated using
    // the standard C++ NEW operator because the transport is responsible for freeing it.
    m_pszAdrType = new TCHAR[lstrlen(pStr)+1];
    if (NULL == m_pszAdrType)
    {
        TraceMessage ("CXPLogon::AddressTypes: Could not allocate memory for address type string");
        hResult = E_OUTOFMEMORY;
    }
    else
    {
        // Copy the ADDRESS TYPE strings
        lstrcpy (m_pszAdrType, pStr);
    
        // Copy address types back to the MAPI pointer
        *pppAdrTypeArray = &m_pszAdrType;
    }
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::RegisterOptions()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      This transport does not registers any per-recipient or per-message
//      option processing, so we return 0 options. And NULL in the OPTIONDATA
//      structure pointer.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CXPLogon::RegisterOptions (ULONG *         pulFlags,
                                        ULONG *         pcOptions,
                                        LPOPTIONDATA *  ppOptions)
{
    InfoTrace ("CXPLogon::RegisterOptions method called");
    CheckParameters (IXPLogon_RegisterOptions, pulFlags);

    // Allocate and initialize the Message & Recipient OptionData structure
    // This memory block is freed in the destructor of the CXPLogon object
    HRESULT hResult = gpfnAllocateBuffer (MAX_NUM_OPTIONS * sizeof(OPTIONDATA), (LPVOID *)&m_pOptData);
    if (hResult)
    {
        TraceResult ("CXPLogon::RegisterOptions: Memory allocation failed", hResult);
        return hResult;
    }

    // The options in for address type, does not do anything, this is
    // just a place holder to show how to specify per-recipient and/or
    // per-message options
    m_pOptData[0].ulFlags        = OPTION_TYPE_MESSAGE;
    m_pOptData[0].lpRecipGUID    = NULL;
    m_pOptData[0].lpszAdrType    = m_pProps[ADDRESS_TYPE].Value.LPSZ;
    m_pOptData[0].lpszDLLName    = TRANSPORT_DLL_NAME_STRING;
    // This is the ordinal value of the callback function as assigned in 
    // the DEF file of this DLL. However, we are not using any 
    // per-message option so this function will not get called by MAPI
    m_pOptData[0].ulOrdinal      = 3;
    m_pOptData[0].cbOptionsData  = 0;
    m_pOptData[0].lpbOptionsData = NULL;
    m_pOptData[0].cOptionsProps  = 0;
    m_pOptData[0].lpOptionsProps = NULL;

    m_pOptData[1].ulFlags        = OPTION_TYPE_RECIPIENT;
    m_pOptData[1].lpRecipGUID    = NULL;
    m_pOptData[1].lpszAdrType    = m_pProps[ADDRESS_TYPE].Value.LPSZ;
    m_pOptData[1].lpszDLLName    = TRANSPORT_DLL_NAME_STRING;
    // This is the ordinal value of the callback function as assigned in 
    // the DEF file of this DLL. However, we are not using any 
    // per-message option so this function will not get called by MAPI
    m_pOptData[1].ulOrdinal      = 3;
    m_pOptData[1].cbOptionsData  = 0;
    m_pOptData[1].lpbOptionsData = NULL;
    m_pOptData[1].cOptionsProps  = 0;
    m_pOptData[1].lpOptionsProps = NULL;

    m_cOptData  = MAX_NUM_OPTIONS;
    
    // Set options on the MAPI-passed pointer
    *pulFlags = fMapiUnicode;
    *pcOptions = MAX_NUM_OPTIONS;    // Number of options
    *ppOptions = m_pOptData;         // Array of actual message options
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::TransportNotify()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Update the status row registered by this transport with MAPI
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CXPLogon::TransportNotify (ULONG * pulFlags, LPVOID * ppvData)
{
    InfoTrace ("CXPLogon::TransportNotify method called");
    CheckParameters (IXPLogon_TransportNotify, pulFlags);

    // In the multithreaded spooler, transports may need to use some kind on 
    // synchonization object. IE. a critical section or a mutex may be 
    // suitable for the purposes here.
    EnterCriticalSection (&(m_pXPObj->m_csTransport));
    ULONG ulOldStatus = m_ulTransportStatus;
    // Set appropriate status flags in our internal state and update the status row, if needed.
    if (*pulFlags & NOTIFY_BEGIN_INBOUND)
    {
        m_ulTransportStatus |= STATUS_INBOUND_ENABLED;
    }
    if (*pulFlags & NOTIFY_END_INBOUND)
    {
        m_ulTransportStatus &= ~STATUS_INBOUND_ENABLED;
    }
    if (*pulFlags & NOTIFY_BEGIN_OUTBOUND)
    {
        m_ulTransportStatus |= STATUS_OUTBOUND_ENABLED;
    }
    if (*pulFlags & NOTIFY_END_OUTBOUND)
    {
        m_ulTransportStatus &= ~STATUS_OUTBOUND_ENABLED;
    }
    if (ulOldStatus != m_ulTransportStatus)
    {
        UpdateStatus();
    }
    LeaveCriticalSection (&(m_pXPObj->m_csTransport));
    return S_OK;
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::Idle()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Stub method. We should not get called here, because we told
//      the spooler not to call us here.
//
//    Return Value
//      S_OK always.
//
STDMETHODIMP CXPLogon::Idle (ULONG ulFlags)
{     
    InfoTrace ("CXPLogon::Idle method called");
    CheckParameters (IXPLogon_Idle, ulFlags);
    return S_OK;
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::TransportLogoff()
//
//    Parameters
//      ulFlags     Priority with which the transport should log off
//
//    Purpose
//      This method is called by the spooler when the transport should do final 
//      arragements before it gets released. In this case it should remove it 
//      self for the linked list of session objects in the CXPProvider object
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CXPLogon::TransportLogoff (ULONG ulFlags)
{
    InfoTrace ("CXPLogon::TransportLogoff method called");
    CheckParameters (IXPLogon_TransportLogoff, ulFlags);

    // Synchronize access to the XP Object since we are going to alter the list of logon object
    EnterCriticalSection (&(m_pXPObj->m_csTransport));

    // Find this session's logon object in the list
    CXPLogon * pPrevLogon = NULL, * pLogon = m_pXPObj->m_pXPSessionList;
    while (pLogon)
    {
        if (this == pLogon)
        {
            // Unlink this session from the list
            if (NULL == pPrevLogon)
            {
                m_pXPObj->m_pXPSessionList = m_pNextSession;
            }
            else
            {
                pPrevLogon->m_pNextSession = m_pNextSession;
            }
            break;
        }
        // Move to the next session
        pPrevLogon = pLogon;
        pLogon = pPrevLogon->m_pNextSession;
    }

    // The pLogon pointer must be non-NULL if we found the session
    HRESULT hResult;
    if (NULL == pLogon)
    {
        TraceMessage ("CXPLogon::TransportLogoff: Invalid pointer to XPLogon object");
        hResult = E_INVALIDARG;
    }
    else
    {
        // We should attempt to remove the transport's status row from
        // the system, but if we fail we won't fail the call.
        hResult = m_pSupObj->ModifyStatusRow (0, NULL, 0);
        TraceResult ("CXPLogon::TransportLogoff: Failed to remove status row", hResult);
        hResult = S_OK; // Get rid of any errors
    }
    // The session list is OK, we can now release the critical section
    LeaveCriticalSection (&m_pXPObj->m_csTransport);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::SubmitMessage()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      This method is called by the spooler when a client submits a 
//      message to a recipient who's address type this transport handles.
//      In this transport, first we check and see if we are not busy in a 
//      previous operation on this session. Then we get a recipient table.
//      We restrict the table for unmarked recipients. After the table is
//      ready we invoke a helper method to process the message (i.e. send it)
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CXPLogon::SubmitMessage (ULONG     ulFlags,
                                      LPMESSAGE pMsgObj, 
                                      ULONG *   pulMsgRef,
                                      ULONG *   pulReturnParm)
{
    InfoTrace ("CXPLogon::SubmitMessage method called");
    CheckParameters (IXPLogon_SubmitMessage, ulFlags);

    #ifdef _DEBUG
    // Before we get called in SubmitMessage or StartMessage, the spooler should have called
    // us in TransportNotify() to let us know which logic (IN, OUT, BOTH) of the
    // transporting mechanism should be activated for this IXPLogon session.
    if (!(m_ulTransportStatus & STATUS_OUTBOUND_ENABLED))
    {
        TraceMessage ("CXPLogon::SubmitMessage: We can't send a message if the outbound is not enabled");
        ASSERTMSG (FALSE, "We can't submit a message if we have not started OUTBOUND logic");
        return MAPI_E_NOT_ME;
    }
    #endif // _DEBUG

    if (m_ulTransportStatus & STATUS_OUTBOUND_ACTIVE)
    {
        TraceMessage ("CXPLogon::SubmitMessage: We can't send a message because we are busy");
        return MAPI_E_BUSY;
    }
    
    // Initialize the pseudo-timer
    CheckSpoolerYield (TRUE);

    // The everybody we are active, sending a message.
    m_ulTransportStatus |= STATUS_OUTBOUND_ACTIVE;
    UpdateStatus();
    
    // Get the recipient table from the message
    LPMAPITABLE pTable = NULL;
    HRESULT hResult = pMsgObj->GetRecipientTable (fMapiUnicode, &pTable);
    if (hResult)
    {
        TraceResult ("CXPLogon::SubmitMessage: Failed to get the recipient table", hResult);
        goto ErrorExit;
    }

    // The spooler marks all the message recipients this transport has to 
    // handle with PR_RESPONSIBILITY set to FALSE
    SPropValue spvRecipUnsent;
    spvRecipUnsent.ulPropTag                       = PR_RESPONSIBILITY;
    spvRecipUnsent.Value.b                         = FALSE;

    SRestriction srRecipientUnhandled;
    srRecipientUnhandled.rt                        = RES_PROPERTY;
	srRecipientUnhandled.res.resProperty.relop     = RELOP_EQ;
	srRecipientUnhandled.res.resProperty.ulPropTag = PR_RESPONSIBILITY;
	srRecipientUnhandled.res.resProperty.lpProp    = &spvRecipUnsent;

    hResult = HrAddColumns (pTable,
                            (LPSPropTagArray)&sptRecipTable,
                            gpfnAllocateBuffer,
                            gpfnFreeBuffer);
    if (hResult)
    {
        TraceResult ("CXPLogon::SubmitMessage: Failed to expand the columns in the recipient table", hResult);
        goto ErrorExit;
    }

    // Let the MAPI spooler do other things
    CheckSpoolerYield();

    LPSRowSet pRecipRows;
    hResult = HrQueryAllRows (pTable,
                              NULL,
                              &srRecipientUnhandled,
                              NULL,
                              0,
                              &pRecipRows);
    if (!hResult)
    {
        // Let the MAPI spooler do other things
        CheckSpoolerYield();
        // Send to the recipients which we can reach, and generate NDR's for the ones we can't
        hResult = SendMailMessage (pMsgObj, pRecipRows);
        FreeProws (pRecipRows);
        if (!hResult)
        {
            // Now we need to save changes on the message and close it
            hResult = pMsgObj->SaveChanges (0);
            TraceResult ("CXPLogon::SubmitMessage: Failed to save the message object", hResult);
        }
    }

ErrorExit:
     // Release the table, we're finished with it
    if (pTable)
    {
        pTable->Release();
    }

    // Release the spooler's message if needed to
    if (pMsgObj)
    {
        pMsgObj->Release ();
    }
    
    // In case there is a warning or error floating around, don't to let it escape to the spooler.
    if (FAILED(hResult))
    {
        // We default to MAPI_E_NOT_ME so that the spooler would attempt handle
        // the message to other transport (currently running in this profile)
        // that handle the same address type as ours.
        TraceMessage ("CXPLogon::SubmitMessage: An error occurred in the submission, returning MAPI_E_NOT_ME");
        hResult = MAPI_E_NOT_ME;
    }
    else
    {
        hResult = S_OK;
    }
    // Notify everybody that we are finished sending the message
    m_ulTransportStatus &= ~STATUS_OUTBOUND_ACTIVE;
    UpdateStatus();
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::EndMessage()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      We don't do anything special here so we just return 0 on the flags.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CXPLogon::EndMessage (ULONG ulMsgRef, ULONG * pulFlags)
{
    InfoTrace ("CXPLogon::EndMessage method called");
    CheckParameters (IXPLogon_EndMessage, ulMsgRef);
    *pulFlags = 0;
    return S_OK;
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::Poll()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Called by the MAPI Spooler to check if there are any messages for the
//      spooler to receive. The method calls the helper function 
//      IsThereAMessageWaiting to check if there is a message file waiting in
//      the directory
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CXPLogon::Poll (ULONG * pulIncoming)
{
    InfoTrace ("CXPLogon::Poll method called");
    CheckParameters (IXPLogon_Poll, pulIncoming);
    if (m_ulTransportStatus & STATUS_INBOUND_ENABLED)
    {
        IsThereAMessageWaiting (*pulIncoming);
    }
    return S_OK;
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::StartMessage()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Called by the spooler after a call to IXPLogon::Poll() reported a
//      pending message. This method first checks for availability of the
//      transport. If busy, it returns with a return error indicating to 
//      try later.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CXPLogon::StartMessage (ULONG      ulFlags,
                                     LPMESSAGE  pMsgObj,
                                     ULONG *    pulMsgRef)
{
    InfoTrace ("CXPLogon::StartMessage method called");
    CheckParameters (IXPLogon_StartMessage, ulFlags);

    // Is the transport available?
    if (m_ulTransportStatus & STATUS_INBOUND_ACTIVE)
    {
        // The transport is busy
        TraceMessage ("CXPLogon::StartMessage: Transport is busy. Try later.");
        return MAPI_E_BUSY;
    }

    // Signal the MAPI subsystem we're receiving a message
    m_ulTransportStatus |= STATUS_INBOUND_ACTIVE;
    UpdateStatus();
    
    LPSTREAM pStream = NULL;
    LPITNEF pTNEFObj = NULL;
    SPropTagArray sptExcludedProps = { 0 };

    // Initialize the pseudo-timer
    CheckSpoolerYield (TRUE);

    // Open the stream where the message properties are.
    HRESULT hResult = OpenStreamOnFile (gpfnAllocateBuffer,
                                        gpfnFreeBuffer,
                		                STGM_READ,
                		                m_szFileFound,
                		                NULL,
                		                &pStream);
    if (hResult)
    {
        TraceResult ("CXPLogon::StartMessage: Failed to open stream on the message file", hResult);
        goto ErrorExit;
    }

    hResult = OpenTnefStream (m_pSupObj,
                              pStream,
                              XP_TNEF_STREAM,
                              TNEF_DECODE,
                              pMsgObj,
                              0,
                              &pTNEFObj);
    if (hResult)
    {
        TraceResult ("CXPLogon::StartMessage: Failed to open TNEF stream object", hResult);
        goto ErrorExit;
    }

    // Let the MAPI spooler do other things
    CheckSpoolerYield();

    // Get the properties for the message which are encoded in the TNEF 
    // message stream. The sptExcludedProps argument, is just a stub, 
    // we are not excluding any properties
    LPSTnefProblemArray pProblems;
    pProblems = NULL;
    hResult = pTNEFObj->ExtractProps (TNEF_PROP_EXCLUDE, &sptExcludedProps, &pProblems);
    if (pProblems)
    {
        gpfnFreeBuffer (pProblems);
        pProblems = NULL;
    }
    if (hResult)
    {
        TraceResult ("CXPLogon::StartMessage: Failed to open TNEF stream", hResult);
        goto ErrorExit;
    }

    // Let the MAPI spooler do other things
    CheckSpoolerYield();

    SetIncomingProps (pMsgObj);
    
    // Process the attachments now, if any
    {
        DWORD dwFileID;
        LPTSTR pStr;
        TCHAR szFileName[MAX_PATH];
        OLECHAR wsFile[MAX_PATH];
        LPSTORAGE pStorage;
        if (!GetFileTitle (m_szFileFound, szFileName, MAX_PATH))
        {
            pStr = m_szFileFound;
            pStr = (LPTSTR)(DWORD)pStr + (sizeof(TCHAR) * lstrlen (MSG_FILENAME_PREFIX));
            pStr = strtok (pStr, TEXT("."));
            ASSERT (pStr);
            dwFileID = atoi (pStr);
            pStr[lstrlen(pStr)] = TEXT('.');
            // Open the compound document data file where the attachments storages and streams are stored
            wsprintfW (wsFile, W_ATT_FILENAME_FORMAT, W_ATT_FILENAME_PREFIX, dwFileID, W_ATT_FILENAME_SUFFIX);
            hResult = StgOpenStorage (wsFile, NULL, XP_OPEN_FLAGS, NULL, 0, &pStorage);
            if (!hResult)
            {
                ReadAttachsFromFile (pMsgObj, pStorage);
                pStorage->Release();
                DeleteFileW (wsFile);
            }
        }
    }

    // Let the MAPI spooler do other things
    CheckSpoolerYield();

    // Save all chages and new properties, back on the message
    hResult = pMsgObj->SaveChanges (0);
    TraceResult ("CXPLogon::StartMessage: Failed to save the message object", hResult);
    
ErrorExit:
    // Release used objects
    if (pTNEFObj)
    {
        pTNEFObj->Release();
    }
    if (pStream)
    {
        pStream->Release();
    }
    DeleteFile (m_szFileFound);
    m_fPendingProcess = FALSE;
    // Tell everyone we are done.
    m_ulTransportStatus &= ~STATUS_INBOUND_ACTIVE;
    UpdateStatus();
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::OpenStatusEntry()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      This method is called to get an IMAPIStatus object for this XPLOGON
//      session.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CXPLogon::OpenStatusEntry (LPCIID          pInterface,
                                        ULONG           ulFlags,
                                        ULONG *         pulObjType,
                                        LPMAPISTATUS *  ppEntry)
{
    InfoTrace ("CXPLogon::OpenStatusEntry method called");
    CheckParameters (IXPLogon_OpenStatusEntry, pInterface);
    if (MAPI_MODIFY & ulFlags)
    {
        TraceMessage ("CXPLogon::OpenStatusEntry: We don't support WRITE access to the status object");
        return E_ACCESSDENIED;
    }

    HRESULT hResult = S_OK;
    // Prevent other threads from accessing this CXPLogon object's CXPProvider parent object
	EnterCriticalSection (&m_pXPObj->m_csTransport);
	// Now, if we already have created a status object on this logon context, 
	// we'll just use QueryInterface to get a new copy (AddRef() it) of the object
    if (!m_pStatusObj)
	{
        // Get the profile section of the PROVIDER so that we may get some properties,
        // assigned by MAPI to this provider, directly on the status object.
        LPPROFSECT pProfileObj = NULL;
        m_pSupObj->OpenProfileSection (NULL, 0, &pProfileObj);
        // If we don't have an object, create it, and save a copy in the logon object
        m_pStatusObj = new CMAPIStatus (this, pProfileObj);
        if (!m_pStatusObj)
        {
            TraceMessage ("CXPLogon::OpenStatusEntry: Could not allocate new CMAPIStatus object");
            hResult = E_OUTOFMEMORY;
        }
        // The constructor of CMAPIStatus AddRef()'ed this object
        if (pProfileObj)
        {
            pProfileObj->Release();
        }
	}
    if (!hResult)
    {
        // The transport will return *ppEntry == NULL or a pointer to the new memory allocated
        m_pStatusObj->AddRef();
        *ppEntry = m_pStatusObj;
        *pulObjType = MAPI_STATUS;
    }
	//	Release the critical section
	LeaveCriticalSection (&m_pXPObj->m_csTransport);
	return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::ValidateState()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      This function gets caller by a client in order to validate the
//      transport logon properties. This function open the profile with the 
//      most up-to-date properties and then compares them to what the transport
//      has stored internally
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CXPLogon::ValidateState (ULONG ulUIParam, ULONG ulFlags)
{
    InfoTrace ("CXPLogon::ValidateState method called");
    CheckParameters (IXPLogon_ValidateState, ulUIParam);

	// Try to open the transport profile section
    LPPROFSECT pProfileObj;
    HRESULT hResult = m_pSupObj->OpenProfileSection (NULL, MAPI_MODIFY, &pProfileObj);
	if (hResult)
	{
		TraceResult ("CXPLogon::ValidateState: IMAPISupport::OpenProfileSection failed", hResult);
		return hResult;
	}

    LPSPropValue pProps = NULL;
    ULONG ulPropCount;
    // Read the properties stored in the profile of this user
	hResult = pProfileObj->GetProps ((LPSPropTagArray)&sptLogonArray, fMapiUnicode, &ulPropCount, &pProps);
	TraceResult ("CXPLogon::ValidateState: Failed to get profile properties", hResult);
    if (!hResult)
    {
    	// Now, compare what the transport thinks the information in the profile is
        // to the real data. If they are different, tell the spooler the transport
        // would like to be reloaded
        if (lstrcmp (m_pProps[DISPLAY_NAME].Value.LPSZ, pProps[DISPLAY_NAME].Value.LPSZ) ||
            lstrcmpi (m_pProps[EMAIL_ADDRESS].Value.LPSZ, pProps[EMAIL_ADDRESS].Value.LPSZ) ||
            lstrcmpi (m_pProps[ADDRESS_TYPE].Value.LPSZ, pProps[ADDRESS_TYPE].Value.LPSZ))
        {
            hResult = m_pSupObj->SpoolerNotify (NOTIFY_CONFIG_CHANGE, NULL);
        	TraceResult ("CXPLogon::ValidateState: Failed to notify the spooler", hResult);
        }
    }
    pProfileObj->Release();
    gpfnFreeBuffer (pProps);
	return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::FlushQueues()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Place holder. Replace this function with actual implementation 
//      if needed.
//
//    Return Value
//      S_OK always;
//
STDMETHODIMP CXPLogon::FlushQueues (ULONG       ulUIParam,
                                    ULONG       cbTargetTransport,
                                    LPENTRYID   pTargetTransport,
                                    ULONG       ulFlags)
{
    InfoTrace ("CXPLogon::FlushQueues method called");
    CheckParameters (IXPLogon_FlushQueues, ulUIParam);
    return S_OK;
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::LoadStatusString()
//
//    Parameters
//      pString         Pointer to a string which will hold the status string
//      uStringSize     Maximum number of characters allowed in the string
//
//    Purpose
//      Loads a string from the transport's stringtable. This method is called 
//      by the CXPLogon::UpdateStatus method when updating a status row. This 
//      method loads the string based on the status flags as indicated in the
//      m_ulTransportStatus. 
//
//    Return Value
//      TRUE            If the string was found in the string table.
//      FALSE           The string was not found. The String indicated by 
//                      pString is set to hold 0 characters
//
BOOL WINAPI CXPLogon::LoadStatusString (LPTSTR pString, UINT uStringSize)
{
    UINT uStringID;
    if (m_ulTransportStatus & STATUS_OUTBOUND_ACTIVE)
    {
        uStringID = IDS_STATUS_UPLOADING;
    }
    else
    {
        if (m_ulTransportStatus & STATUS_INBOUND_ACTIVE)
        {
            uStringID = IDS_STATUS_DOWNLOADING;
        }
        else 
        {
            if ((m_ulTransportStatus & STATUS_AVAILABLE) &&
                ((m_ulTransportStatus & STATUS_INBOUND_ENABLED) ||
                (m_ulTransportStatus & STATUS_OUTBOUND_ENABLED)))
            {
                uStringID = IDS_STATUS_ONLINE;
            }
            else
            {
                if(m_ulTransportStatus & STATUS_AVAILABLE)
                {
                    uStringID = IDS_STATUS_AVAILABLE;
                }
                else
                {
                    uStringID = IDS_STATUS_OFFLINE;
                }
            }
        }
    }
    if (LoadString (m_hInstance, uStringID, pString, uStringSize))
    {
        return TRUE;
    }
    pString[0] = '\0';
    return FALSE;
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::MakeSearchKey()
//
//    Parameters
//      pParentMemBlock     Pointer the a block of memory allocated with the
//                          MAPI Mem. alloc. functions.
//      pszAddrType         String with the address type
//      pszAddress          String with the actual mail address
//      pcbSearchKey        Pointer to a ULONG which returns the size
//                          (in bytes) of the newly created search key
//      ppSearchKey         Pointer to an array ob BYTES
//
//    Purpose
//      Returns a MAPI search key of the form TYPE:ADDRESS in upper case. 
//      The string is allocated with the MAPI Mem Alloc functions and linked
//      to the block of memory indicated by pParentMemBlock
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CXPLogon::MakeSearchKey (LPVOID        pParentMemBlock,
                                      LPTSTR        pszAddrType,
                                      LPTSTR        pszAddress,
                                      ULONG *       pcbSearchKey,
                                      LPBYTE *      ppSearchKey)
{
    LPTSTR pszSearchKey;
    // The 2 is for the COLON and the NULL terminator
    ULONG cbStrSize = sizeof (TCHAR) * (2 + lstrlen (pszAddrType) + lstrlen (pszAddress));
    HRESULT hResult = gpfnAllocateMore (cbStrSize, pParentMemBlock, (LPVOID *)&pszSearchKey);
    TraceResult ("CXPLogon::MakeSearchKey: Failed to allocate search key string", hResult);
    if (!hResult)
    {
        // We need to convert the address to upper case
        wsprintf (pszSearchKey, TEXT("%s:%s"), pszAddrType, pszAddress);
        CharUpper (pszSearchKey);
        *pcbSearchKey = cbStrSize;
        *ppSearchKey = (LPBYTE)pszSearchKey;
    }
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::UpdateStatus()
//
//    Parameters
//      None
//
//    Purpose
//      Updates the transport status row of this transport in the MAPI Mail
//      subsystem. Updates the flags according the internal state flags 
//      maintained in m_ulTransportStatus and loads the readable status string
//      to reset the status row. The caller of this method should update 
//      m_ulTransportStatus member variable prior to calling UpdateStatus()
//
//    Return Value
//      None
//
void WINAPI CXPLogon::UpdateStatus ()
{
    ULONG cProps = 1;
    SPropValue rgProps[2] = { 0 };
    //  Store the new Transport Provider Status Code
    rgProps[0].ulPropTag = PR_STATUS_CODE;
    rgProps[0].Value.ul = m_ulTransportStatus;

    TCHAR szStatus[64];
    // Set the Status String according to m_ulTransportStatus
    if  (LoadStatusString (szStatus, sizeof(szStatus) - 1 ))
    {
        rgProps[1].ulPropTag = PR_STATUS_STRING;
        rgProps[1].Value.LPSZ = szStatus;
        cProps = 2;
    }
    //  OK. Notify the messaging subsystem of state changes in this transport.
    HRESULT hResult = m_pSupObj->ModifyStatusRow (cProps, rgProps, STATUSROW_UPDATE);
    TraceResult ("CXPLogon::UpdateStatus: ModifyStatusRow failed", hResult);
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::InitializeTransportStatusFlags()
//
//    Parameters
//      ulFlags     Flags passed in to initialize the transport status flags.
//
//    Purpose
//      Initialize the transport status flags with the flags passed in by the 
//      MAPI spooler. This method gets called when initializing a CXPLogon 
//      object in the CXPProvider::TransportLogon method.
//
//    Return Value
//      None
//
void WINAPI CXPLogon::InitializeTransportStatusFlags (ULONG ulFlags)
{
    if (ulFlags & LOGON_NO_CONNECT)
    {
        m_ulTransportStatus = STATUS_OFFLINE;
    }
    else
    {
        m_ulTransportStatus = STATUS_AVAILABLE;
        if (!(ulFlags & LOGON_NO_INBOUND))
        {
            m_ulTransportStatus |= STATUS_INBOUND_ENABLED;
        }
        if (!(ulFlags & LOGON_NO_OUTBOUND))
        {
            m_ulTransportStatus |= STATUS_OUTBOUND_ENABLED;
        }
    }
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::InitializePropArray()
//
//    Parameters
//      None
//
//    Purpose
//      Initializes the transport ID array with the properties found in the
//      profile for the user
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CXPLogon::InitializePropArray()
{
    // Allocate the property array for transport. This memory block is freed 
    // in the destructor of the CXPLogon object
    HRESULT hResult = gpfnAllocateBuffer (sizeof(SPropValue)*NUM_SENDER_PROPS, (LPVOID *)&m_pIdentityProps);
    if (hResult)
    {
        TraceResult ("CXPLogon::InitializePropArray: Failed to allocate property array", hResult);
        return hResult;
    }
    
    ///////////////////////////////////////////////////////////////////////////
    // Set the PR_SENDER_ENTRYID property. Create an entry for the user and
    // set it in the property array
    m_pIdentityProps[XPID_EID].ulPropTag = PR_SENDER_ENTRYID;
    hResult = m_pSupObj->CreateOneOff (m_pProps[DISPLAY_NAME].Value.LPSZ,
                                       m_pProps[ADDRESS_TYPE].Value.LPSZ,
                                       m_pProps[EMAIL_ADDRESS].Value.LPSZ,
                                       fMapiUnicode,
                                       &m_pIdentityProps[XPID_EID].Value.bin.cb,
                                       (LPENTRYID *)&m_pIdentityProps[XPID_EID].Value.bin.lpb);
    if (hResult)
    {
        TraceResult ("CXPLogon::InitializePropArray: Failed to create a one off identity", hResult);
        m_pIdentityProps[XPID_EID].ulPropTag = PROP_TAG (PT_ERROR, PROP_ID (PR_SENDER_ENTRYID));
        return hResult;
    }
    //////////////////////////////////////////////////////////////////////////
    // Set the PR_SENDER_NAME property
    m_pIdentityProps[XPID_NAME].ulPropTag = PR_SENDER_NAME;
    LPTSTR pStr1, pStr2 = m_pProps[DISPLAY_NAME].Value.LPSZ;
    hResult = gpfnAllocateMore (Cbtszsize(pStr2), m_pIdentityProps, (LPVOID *)&pStr1);
    if (hResult)
    {
        TraceResult ("CXPLogon::InitializePropArray: Memory allocation failed for display name string", hResult);
        m_pIdentityProps[XPID_NAME].ulPropTag = PROP_TAG (PT_ERROR, PROP_ID (PR_SENDER_NAME));
        return hResult;
    }
    lstrcpy (pStr1, pStr2);
    m_pIdentityProps[XPID_NAME].Value.LPSZ = pStr1;

    //////////////////////////////////////////////////////////////////////////
    // Set the PR_SENDER_SEARCH_KEY property
    m_pIdentityProps[XPID_SEARCH_KEY].ulPropTag = PR_SENDER_SEARCH_KEY;
    // First, create the search key
    hResult = MakeSearchKey (m_pIdentityProps,
                             m_pProps[ADDRESS_TYPE].Value.LPSZ,
                             m_pProps[EMAIL_ADDRESS].Value.LPSZ,
                             &m_pIdentityProps[XPID_SEARCH_KEY].Value.bin.cb,
                             &m_pIdentityProps[XPID_SEARCH_KEY].Value.bin.lpb);
    if (hResult)
    {
        m_pIdentityProps[XPID_SEARCH_KEY].ulPropTag = PROP_TAG (PT_ERROR, PROP_ID (PR_SENDER_SEARCH_KEY));
    }
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::SetSessionFlags()
//
//    Parameters
//      [IN, OUT]   pulFlags    Flags returned to caller
//
//    Purpose
//      To set the flags we will return to the spooler during the CXPLogon 
//      initialization in the CXPProvider::TransportLogon method. The ulFlags
//      parameter is filled as a return value to the caller
//
//    Return Value
//      None
//
void WINAPI CXPLogon::SetSessionFlags (ULONG * pulFlags)
{
    if (*pulFlags & LOGON_NO_CONNECT)
    {
        // Nothing is set
        *pulFlags = 0;
    }
    else
    {
        // We DON'T want the MAPI spooler to call our IXPLogon::Idle method (LOGON_SP_IDLE not added)
        // We want the MAPI spooler to call our IXPLogon::Poll method (LOGON_SP_POLL added)
        // Also the names have to be fully resolved (LOGON_SP_RESOLVE)
        *pulFlags = LOGON_SP_RESOLVE | LOGON_SP_POLL;
    }
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::SendMailMessage()
//
//    Parameters
//      pMsgObj     Pointer to an IMessage object passed to us by the
//                  MAPI spooler.
//      pRecipRows  Row set with ALL the rows from the recipient table which
//                  have the properties of the recipients we are sending the
//                  message to.
//
//    Purpose
//      This method gets called by SubmitMessage() to do the delivery of the
//      message to the list of recipients. Each recipient will have the 
//      PR_RESPONSIBILITY property set indicating weather or not message 
//      reached the recipient.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CXPLogon::SendMailMessage (LPMESSAGE   pMsgObj,
                                        LPSRowSet   pRecipRows)
{
    LPADRLIST pAdrList = NULL, pAdrListFailed = NULL;
    LPSTnefProblemArray pProblems = NULL;
    TCHAR szMessageFile[MAX_PATH];
    DWORD dwFileID;
    ULONG ulCount1 = 0, ulCount2 = 0, ulRow;
    BOOL fSentSuccessfully;
    LPSPropValue pProps;

    if (!GetMsgFileName (szMessageFile, dwFileID))
    {
        return E_FAIL;
    }
    // Create a stream where all message information will be saved.
    LPSTREAM pStream;
    HRESULT hResult = OpenStreamOnFile (gpfnAllocateBuffer,
                                        gpfnFreeBuffer,
        		                        STGM_CREATE | STGM_READWRITE,
        		                        szMessageFile,
        		                        NULL,
        		                        &pStream);
	if (hResult)
    {
        TraceResult ("CXPLogon::SendMailMessage: Failed to create stream object", hResult);
        return hResult;
    }

    // The wKey is a key used to identify the TNEF property stream. Transports
    // should generate a pseudo-random number for this field. Here we get
    // a number based upon the system's tic count
    // Note that this number cannot be zero of the  OpenTnefStream call will be fail.
    WORD wKey = LOWORD (GetTickCount());
    if (!wKey)
    {
        ASSERTMSG (FALSE, "I'll be darn! It's zero!!!");
        wKey = LOWORD (GetTickCount()) + 1;
        ASSERTMSG (0 != wKey, "No way! What is going on!?!?");
    }
    LPITNEF pTNEFObj = NULL;
    hResult = OpenTnefStream (m_pSupObj,
                              pStream,
                              XP_TNEF_STREAM,
                              TNEF_ENCODE | TNEF_PURE,
                              pMsgObj,
                              wKey,
                              &pTNEFObj);
    if (hResult)
    {
        TraceResult ("CXPLogon::SendMailMessage: Failed to create TNEF object", hResult);
        goto ErrorExit;
    }
    // Let the MAPI spooler do other things
    CheckSpoolerYield();

    // Get the current time. We need to set some properties that require the time
    SYSTEMTIME st;
    FILETIME ft;
    GetSystemTime (&st);
    SystemTimeToFileTime (&st, &ft);

    BOOL fNeedDeliveryReport, fHasAttachments;
    SetOutgoingProps (pMsgObj, ft, fNeedDeliveryReport, fHasAttachments);
    if (fHasAttachments)
    {
        // This is where this transport handles the attachment in a "Custom" manner
        // We handle attachments by value whose data is in PR_ATTACH_DATA_BIN. The
        // data is save to a separate data file that is sent along with the message
        hResult = WriteAttachsToFile (pMsgObj, dwFileID);
        if (hResult)
        {
            goto ErrorExit;
        }
    }
    // Let the MAPI spooler do other things
    CheckSpoolerYield();
    
    // Check what properties there are and exclude the Non-Transmittables ones
    LPSPropTagArray pTags;
    hResult = pMsgObj->GetPropList (fMapiUnicode, &pTags);
	if (hResult)
	{
		TraceResult ("CXPLogon::SendMailMessage: Failed to get the message property tags", hResult);
        goto ErrorExit;
	}

    // In this sample transport we opted to let TNEF encapsulate
    // the table of recipients of this message. This has a side effect: The
    // addresses of the recipients get munged into the TNEF stream. So, for
    // example, if the TNEF file goes through a foreign mail system and it
    // needs to translate the address of the recipients, it will not be
    // able to. A tranport using TNEF which trasmits messages to a foreign
    // mail system, must do custom processing of the recipients and their
    // address so the recieving side will understands them.
    // This sample code must be modified if you want it as the base code
    // for a gateway transport.
    // Here we also let TNEF encode all the attachments we want to send.
	ULONG cValues, i;
    cValues = 0;
    for (i=0; i<pTags->cValues; i++)
	{
		// Use the FIsTransmittable macro in MAPI to determine if a 
		// property is transmittable or not.
		if ((FIsTransmittable(pTags->aulPropTag[i]) ||
		     PR_MESSAGE_RECIPIENTS == pTags->aulPropTag[i]) &&
             PR_MESSAGE_ATTACHMENTS != pTags->aulPropTag[i])
		{
			pTags->aulPropTag[cValues++] = pTags->aulPropTag[i];
		}
    }
    pTags->cValues = cValues;

    // Encode the properties now
    hResult = pTNEFObj->AddProps (TNEF_PROP_MESSAGE_ONLY | TNEF_PROP_INCLUDE, 0, NULL, pTags);
    gpfnFreeBuffer (pTags);
    if (FAILED(hResult)) // There could be warnings
	{
		TraceResult ("CXPLogon::SendMailMessage: Failed to Add the TNEF properties to the message stream", hResult);
        goto ErrorExit;
	}
    // Let the MAPI spooler do other things
    CheckSpoolerYield();


    if (fHasAttachments)
    {
        EncodeAttachments (pTNEFObj, pMsgObj);
    }
    // Let the MAPI spooler do other things
    CheckSpoolerYield();


    hResult = pTNEFObj->Finish (0, &wKey, &pProblems);
    if (FAILED(hResult)) // There could be warnings
    {
        TraceResult ("CXPLogon::SendMailMessage: Failed to save the TNEF props", hResult);
        goto ErrorExit;
    }
    if (pProblems)
    {
        gpfnFreeBuffer (pProblems);
        pProblems = NULL;
        TraceMessage ("CXPLogon::SendMailMessage: Some problem(s) occurred while finishing the TNEF encoding");
        // Don't have to fail submission
	}

    // Release used objects and NULL the object pointers to avoid re-releasing them during cleanup
    if (pTNEFObj)
    {
        pTNEFObj->Release();
        pTNEFObj = NULL;
    }
    if (pStream)
    {
        pStream->Release();
        pStream = NULL;
    }
    // Let the MAPI spooler do other things
    CheckSpoolerYield();

    // Traverse the row set
    for (ulRow=0; ulRow<pRecipRows->cRows; ulRow++)
    {
        pProps = pRecipRows->aRow[ulRow].lpProps;

        fSentSuccessfully = TRUE;  // Modify this flag to cause NDR's
        
        // Set the PR_RESPONSIBILITY flag to indicated we have handled this
        // recipient (row). If the flag is not modified, MAPI will pass this
        // message to the next transport in the profile that knows how to
        // handle the address types this transport knows.
        // In this case WE want to be the last to handle the message. If
        // we fail the submission, we should tell MAPI to generate an NDR.
        pProps[RECIP_RESPONSIBILITY].ulPropTag = PR_RESPONSIBILITY;
        pProps[RECIP_RESPONSIBILITY].Value.b = TRUE;

        // Set the report time for DR's and NDR's
        pProps[RECIP_REPORT_TIME].ulPropTag = PR_REPORT_TIME;
        pProps[RECIP_REPORT_TIME].Value.ft = ft;

        if (!fSentSuccessfully)
        {
            // The PR_NULL value in the PR_ROWID causes the entry to 
            // generate an NDR in the spooler.
            pProps[RECIP_ROWID].ulPropTag = PR_NULL;

            // Make the spooler generate an NDR instead of DR
            pProps[RECIP_DELIVER_TIME].ulPropTag = PR_NULL;

            // The Spooler will generate an NDR report and will fill in
            // all required properties in the StatusRecips call. The only
            // thing we need to do is to fill in a specific per-recipient
            // text description of the problem. It's good to have real 
            // info from the transport indicating the real cause for the
            // failure
            TCHAR szReportText[256], szBuffer[512];
            LPTSTR pStr;
            LoadString (m_hInstance,
                        IDS_REPORT_TEXT_MSG, 
                        szReportText,
                        255);
            wsprintf (szBuffer,
                      TEXT("%s    Address: %s"),
                      szReportText,
                      pProps[RECIP_EMAIL_ADR].Value.LPSZ);
            hResult = gpfnAllocateMore (Cbtszsize(szBuffer), pProps, (LPVOID *)&pStr);
            if (!hResult)
            {
                // Copy the formatted string and hook it into the
                // pre-allocated (by MAPI) column
                lstrcpy (pStr, szBuffer);
                pProps[RECIP_REPORT_TEXT].ulPropTag = PR_REPORT_TEXT;
                pProps[RECIP_REPORT_TEXT].Value.LPSZ = pStr;
            }
            else
            {
                pProps[RECIP_REPORT_TEXT].ulPropTag = PROP_TAG (PT_ERROR, PROP_ID (PR_REPORT_TEXT));
                pProps[RECIP_REPORT_TEXT].Value.err = hResult;
                TraceResult ("CXPLogon::SendMailMessage: memory allocation failed for the NDR report string", hResult);
            }
        }
        else
        {
            // For delivery report, each recipient must have this property set.
            // Otherwise the spooler will default to generate an NDR instead.
            pProps[RECIP_DELIVER_TIME].ulPropTag = PR_DELIVER_TIME;
            pProps[RECIP_DELIVER_TIME].Value.ft = ft;

            pProps[RECIP_REPORT_TEXT].ulPropTag = PROP_TAG (PT_ERROR, PROP_ID (PR_REPORT_TEXT));
            pProps[RECIP_REPORT_TEXT].Value.err = S_OK;
        }

        // Based on the result of the submission to the remote host we determine into
        // which address list to add this recipient
        LPADRLIST * ppTmpList = (fSentSuccessfully ? &pAdrList : &pAdrListFailed);
        ULONG ulTmpCount = (fSentSuccessfully ? ulCount1 : ulCount2);

        // Let the MAPI spooler do other things
        CheckSpoolerYield();

        // Does the list where this recipient goes have enough room for one more entry?
        // If not, resize the address list to hold QUERY_SIZE more entries.
        if (!(*ppTmpList) || ((*ppTmpList)->cEntries + 1 > ulTmpCount))
        {
            hResult= GrowAdrList (ppTmpList, 10, ulTmpCount);
            if (hResult)
            {
                goto ErrorExit;
            }
            ulCount1 = (fSentSuccessfully ? ulTmpCount : ulCount1);
            ulCount2 = (!fSentSuccessfully ? ulTmpCount : ulCount2);
        }

        // We have room now so store the new ADRENTRY. As part of the
        // storage, we're going to copy the SRow pointer from the SRowSet
        // into the ADRENTRY. Once we've done this, we won't need the
        // SRowSet any more ... and the SRow will be released when
        // we unwind the ADRLIST
        (*ppTmpList)->aEntries[(*ppTmpList)->cEntries].cValues = pRecipRows->aRow[ulRow].cValues;
        (*ppTmpList)->aEntries[(*ppTmpList)->cEntries].rgPropVals = pRecipRows->aRow[ulRow].lpProps;

        // Increase the number of entries in the address list
        (*ppTmpList)->cEntries++;

        // Now that we are finished with this row (it is in the right
        // adrlist) we want to disassociate it from the rowset
        // so we don't delete this before we modify the recipients list
        pRecipRows->aRow[ulRow].lpProps = NULL;
        pRecipRows->aRow[ulRow].cValues = 0;
    }
    
    // Let the MAPI spooler do other things
    CheckSpoolerYield();

    // Do we have some recipients that the message arrived to?
    if (pAdrList)
    {
        hResult = pMsgObj->ModifyRecipients (MODRECIP_MODIFY, pAdrList);
        if (hResult)
        {
            TraceResult ("CXPLogon::SendMailMessage: ModifyRecipients failed", hResult);
        }
        if (fNeedDeliveryReport)
        {
            hResult = m_pSupObj->StatusRecips (pMsgObj, pAdrList);
            if (hResult)
            {
                TraceResult ("CXPLogon::SendMailMessage: StatusRecips (DR) failed", hResult);
            }
            // If we were successful, we should null out the pointer becase MAPI released
            // the memory for this structure. And we should not try to release it
            // again in the cleanup code.
            pAdrList = NULL;
        }
    }
    // Do we have some recipients that the message DID NOT arrived to?
    if (pAdrListFailed)
    {
        // The address list has the entries with the PR_RESPONSIBILITY set, so the
        // spooler will know if it has to generate NDR reports.
        hResult = m_pSupObj->StatusRecips (pMsgObj, pAdrListFailed);
        if (hResult)
        {
            TraceResult ("CXPLogon::SendMailMessage: StatusRecips (NDR) failed", hResult);
        }
        // If we were successful, we should null out the pointer becase MAPI released
        // the memory for this structure. And we should not try to release it
        // again in the cleanup code.
        pAdrListFailed = NULL;
    }

ErrorExit:
    gpfnFreeBuffer (pProblems);
    gpfnFreeBuffer (pAdrList);
    gpfnFreeBuffer (pAdrListFailed);

    // Release used objects
    if (pTNEFObj)
    {
        pTNEFObj->Release();
    }
    if (pStream)
    {
        pStream->Release();
    }
    if (hResult)
    {
        DeleteFile (szMessageFile);
        wsprintf (szMessageFile,
                  ATT_FILENAME_FORMAT,
                  ATT_FILENAME_PREFIX,
                  dwFileID,
                  ATT_FILENAME_SUFFIX);
        DeleteFile (szMessageFile);
    }
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::SetOutgoingProps()
//
//    Parameters
//      pMsgObj             Pointer to the IMessage object we are processing.
//      ft                  FILETIME structure with the time at we the message
//                          is being delivered.
//      fNeedDeliveryReport We return TRUE if the sender requested delivery
//                          reports. FALSE otherwise
//      fHasAttachments     We return TRUE if the message has attachments,
//                          FALSE otherwise.
//
//    Purpose
//      This function checks that the sender information is on the message
//      so that we may get replies from the intended recipients.
//      
//    Return Value
//      None.
//
void WINAPI CXPLogon::SetOutgoingProps (LPMESSAGE   pMsgObj,
                                        FILETIME    ft,
                                        BOOL &      fNeedDeliveryReport,
                                        BOOL &      fHasAttachments)
{
    // Default return values in case of a failure
    fNeedDeliveryReport = FALSE;
    fHasAttachments = TRUE;

    LPSPropValue pSender;
    ULONG ulValues;
    HRESULT hResult = pMsgObj->GetProps ((LPSPropTagArray)&sptOutMsgProps, fMapiUnicode, &ulValues, &pSender);
    if (FAILED(hResult))
    {
        TraceResult ("CXPLogon::SetOutgoingProps: Failed to get properties from the message", hResult);
        pSender = NULL; // So that we may recover and continue using default values
    }
    ASSERT (OUT_MSG_PROPS == ulValues);
    SPropValue spvProps[7] = { 0 };
    ULONG i = 0;
    // If no sender has been stamped on the message use the identity of the transport
    if (!pSender || PR_SENDER_ENTRYID != pSender[0].ulPropTag)
    {
        spvProps[i].ulPropTag = PR_SENDER_NAME;
	    spvProps[i++].Value.LPSZ = m_pProps[DISPLAY_NAME].Value.LPSZ;

        spvProps[i].ulPropTag = PR_SENDER_EMAIL_ADDRESS;
	    spvProps[i++].Value.LPSZ = m_pProps[EMAIL_ADDRESS].Value.LPSZ;

        spvProps[i].ulPropTag = PR_SENDER_ADDRTYPE;
	    spvProps[i++].Value.LPSZ = m_pProps[ADDRESS_TYPE].Value.LPSZ;

        spvProps[i].ulPropTag = PR_SENDER_ENTRYID;
	    spvProps[i++].Value.bin = m_pIdentityProps[XPID_EID].Value.bin;

        spvProps[i].ulPropTag = PR_SENDER_SEARCH_KEY;
	    spvProps[i++].Value.bin = m_pIdentityProps[XPID_SEARCH_KEY].Value.bin;
    }
    // The MS Exchange mail viewer requires this property
    if (!pSender || PR_SENT_REPRESENTING_NAME != pSender[1].ulPropTag)
    {
        spvProps[i].ulPropTag = PR_SENT_REPRESENTING_NAME;
        spvProps[i++].Value.LPSZ = m_pProps[DISPLAY_NAME].Value.LPSZ;
    }
    if (PR_ORIGINATOR_DELIVERY_REPORT_REQUESTED == pSender[2].ulPropTag)
    {
        if (pSender[2].Value.b)
        {
            fNeedDeliveryReport = TRUE;
        }
        else
        {
            fNeedDeliveryReport = FALSE;
        }
    }
    if (PR_HASATTACH == pSender[3].ulPropTag)
    {
        if (pSender[3].Value.b)
        {
            fHasAttachments = TRUE;
        }
        else
        {
            fHasAttachments = FALSE;
        }
    }

    gpfnFreeBuffer (pSender);

    // Set the time when this transport actually transmitted the message
    spvProps[i].ulPropTag = PR_MESSAGE_DELIVERY_TIME;
    spvProps[i++].Value.ft = ft;
    spvProps[i].ulPropTag = PR_PROVIDER_SUBMIT_TIME;
    spvProps[i++].Value.ft = ft;

    hResult = pMsgObj->SetProps (i, spvProps, NULL);
    TraceResult ("CXPLogon::SetOutgoingProps: Failed to set properties in the message", hResult);
    if (!hResult)
    {
        // We need to keep the message open because we are not finished using it.
        hResult = pMsgObj->SaveChanges (KEEP_OPEN_READWRITE);
        TraceResult ("CXPLogon::SetOutgoingProps: SaveChanges failed", hResult);
    }
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::SetIncomingProps()
//
//    Parameters
//      pMsgObj     Pointer to the IMessage object we are processing.
//
//    Purpose
//      This function sets the PR_SENT_REPRESENTING_xxx properties in an
//      incoming message if those properties aren't already in the message.
//      For these properties we use the values for the sender of the message.
//      We also walk the recipient list restricted to the address type of our
//      transport looking for us, the recipient of the message, s that we may
//      set the PR_MESSAGE_TO_xxx and PR_MESSAGE_RECIP_ME properties in
//      the message.
//
//    Return Value
//      None.
//
void WINAPI CXPLogon::SetIncomingProps (LPMESSAGE pMsgObj)
{
    SPropValue spvProps[27] =  { 0 };
    ULONG ulValues, i = 0;
    LPSPropValue pProps;
    HRESULT hResult = pMsgObj->GetProps ((LPSPropTagArray)&sptNewMsgProps, fMapiUnicode, &ulValues, &pProps);
    if (FAILED(hResult))
    {
        TraceResult ("CXPLogon::SetIncomingProps: Failed to get the message changes", hResult);
        pProps = NULL;
        goto RecoverAndContinue;
    }
    ASSERT (sptNewMsgProps.cValues == ulValues);
    // This properties should ALWAYS be in an incoming message
    ASSERT (PR_SENDER_NAME == pProps[NEW_SENDER_NAME].ulPropTag);
    ASSERT (PR_SENDER_EMAIL_ADDRESS == pProps[NEW_SENDER_EMAIL].ulPropTag);

    LPSPropValue pSenderSKEY, pSenderEID;
    // We need to make a search key for this recipient.
    spvProps[i].ulPropTag = PR_SENDER_SEARCH_KEY;
    hResult = MakeSearchKey (pProps,
                             m_pProps[ADDRESS_TYPE].Value.LPSZ,
                             pProps[NEW_SENDER_EMAIL].Value.LPSZ,
                             &spvProps[i].Value.bin.cb,
                             &spvProps[i].Value.bin.lpb);
    if (!hResult)
    {
        pSenderSKEY = &spvProps[i];
        i++;
        // We need to create a one-off entry in the address book so that we may
        // reply to this recipient. If the recipient already exist, its
        // entry Id will be returned instead.
        spvProps[i].ulPropTag = PR_SENDER_ENTRYID;
        hResult = m_pSupObj->CreateOneOff (pProps[NEW_SENDER_NAME].Value.LPSZ,
                                           m_pProps[ADDRESS_TYPE].Value.LPSZ,
                                           pProps[NEW_SENDER_EMAIL].Value.LPSZ,
                                           fMapiUnicode,
                                           &spvProps[i].Value.bin.cb,
                                           (LPENTRYID *)&spvProps[i].Value.bin.lpb);
        if (hResult)
        {
            i--; // Ignore this property and the previous
        }
        else
        {
            pSenderEID = &spvProps[i];
            i++;
        }
    }
    TraceResult ("CXPLogon::SetIncomingProps: Failed to create a oneoff for the sender", hResult);

    if (PR_SENT_REPRESENTING_NAME == pProps[NEW_SENT_NAME].ulPropTag &&
        PR_SENT_REPRESENTING_ADDRTYPE == pProps[NEW_SENT_ADRTYPE].ulPropTag &&
        PR_SENT_REPRESENTING_EMAIL_ADDRESS == pProps[NEW_SENT_EMAIL].ulPropTag)
    {
        spvProps[i].ulPropTag = PR_SENT_REPRESENTING_SEARCH_KEY;
        hResult = MakeSearchKey (pProps,
                                 pProps[NEW_SENT_ADRTYPE].Value.LPSZ,
                                 pProps[NEW_SENT_EMAIL].Value.LPSZ,
                                 &spvProps[i].Value.bin.cb,
                                 &spvProps[i].Value.bin.lpb);
        if (!hResult)
        {
            i++;
            spvProps[i].ulPropTag = PR_SENT_REPRESENTING_ENTRYID;
            hResult = m_pSupObj->CreateOneOff (pProps[NEW_SENT_NAME].Value.LPSZ,
                                               pProps[NEW_SENT_ADRTYPE].Value.LPSZ,
                                               pProps[NEW_SENT_EMAIL].Value.LPSZ,
                                               fMapiUnicode,
                                               &spvProps[i].Value.bin.cb,
                                               (LPENTRYID *)&spvProps[i].Value.bin.lpb);
            if (hResult)
            {
                i--; // Ignore this property and the previous
            }
            else
            {
                i++;
            }
        }
        // We trace the error but is not fatal
        TraceResult ("CXPLogon::SetIncomingProps: Failed to create a oneoff for the delegated sender", hResult);
    }
    else
    {
        // If the message doesn't have PR_SENT_REPRESENTING_xxx properties AND we were
        // successful at creating the one-off for the sender, we set PR_SENT_REPRESENTING_xxx
        // to the same values as PR_SENDER_xxx
        if (!hResult)
        {
            spvProps[i].ulPropTag       = PR_SENT_REPRESENTING_NAME;
            spvProps[i++].Value.LPSZ    = pProps[NEW_SENDER_NAME].Value.LPSZ;
            spvProps[i].ulPropTag       = PR_SENT_REPRESENTING_ADDRTYPE;
            spvProps[i++].Value.LPSZ    = m_pProps[ADDRESS_TYPE].Value.LPSZ;
            spvProps[i].ulPropTag       = PR_SENT_REPRESENTING_NAME;
            spvProps[i++].Value.LPSZ    = pProps[NEW_SENDER_NAME].Value.LPSZ;
            spvProps[i].ulPropTag       = PR_SENT_REPRESENTING_EMAIL_ADDRESS;
            spvProps[i++].Value.LPSZ    = pProps[NEW_SENDER_EMAIL].Value.LPSZ;
        
            ASSERT (PR_SENDER_ENTRYID == pSenderEID->ulPropTag);
            spvProps[i].ulPropTag       = PR_SENT_REPRESENTING_ENTRYID;
    	    spvProps[i++].Value         = pSenderEID->Value;

            ASSERT (PR_SENDER_SEARCH_KEY == pSenderSKEY->ulPropTag);
            spvProps[i].ulPropTag       = PR_SENT_REPRESENTING_SEARCH_KEY;
    	    spvProps[i++].Value         = pSenderSKEY->Value;
        }
    }

    // If the sender's address is the same as our address (the recipient), then the message
    // was sent from us, to ourselves, therefore we must set the flag bit in the message.
    if (0 == lstrcmpi (pProps[NEW_SENDER_EMAIL].Value.LPSZ, m_pProps[EMAIL_ADDRESS].Value.LPSZ))
    {
        ASSERT (PR_MESSAGE_FLAGS == pProps[NEW_MSG_FLAGS].ulPropTag);
        spvProps[i].ulPropTag = PR_MESSAGE_FLAGS;
        spvProps[i++].Value.l = pProps[NEW_MSG_FLAGS].Value.l | MSGFLAG_FROMME;
    }

RecoverAndContinue:
    
    // We must stamp the message with this properties since we (this transport)
    // were the ones who processed the incoming message.
    spvProps[i].ulPropTag       = PR_RECEIVED_BY_ENTRYID;
    spvProps[i++].Value.bin     = m_pIdentityProps[XPID_EID].Value.bin;
    spvProps[i].ulPropTag       = PR_RECEIVED_BY_NAME;
	spvProps[i++].Value.LPSZ    = m_pIdentityProps[XPID_NAME].Value.LPSZ;
    spvProps[i].ulPropTag       = PR_RECEIVED_BY_SEARCH_KEY;
	spvProps[i++].Value.bin     = m_pIdentityProps[XPID_SEARCH_KEY].Value.bin;
    spvProps[i].ulPropTag       = PR_RECEIVED_BY_ADDRTYPE;
    spvProps[i++].Value.LPSZ    = m_pProps[ADDRESS_TYPE].Value.LPSZ;
    spvProps[i].ulPropTag       = PR_RECEIVED_BY_EMAIL_ADDRESS;
    spvProps[i++].Value.LPSZ    = m_pProps[EMAIL_ADDRESS].Value.LPSZ;

    // In this transport we set PR_RCVD_xxx with the values of PR_RECEIVED_xxx
    spvProps[i].ulPropTag       = PR_RCVD_REPRESENTING_ENTRYID;
    spvProps[i++].Value.bin     = m_pIdentityProps[XPID_EID].Value.bin;
    spvProps[i].ulPropTag       = PR_RCVD_REPRESENTING_NAME;
	spvProps[i++].Value.LPSZ    = m_pIdentityProps[XPID_NAME].Value.LPSZ;
    spvProps[i].ulPropTag       = PR_RCVD_REPRESENTING_SEARCH_KEY;
	spvProps[i++].Value.bin     = m_pIdentityProps[XPID_SEARCH_KEY].Value.bin;
    spvProps[i].ulPropTag       = PR_RCVD_REPRESENTING_ADDRTYPE;
    spvProps[i++].Value.LPSZ    = m_pProps[ADDRESS_TYPE].Value.LPSZ;
    spvProps[i].ulPropTag       = PR_RCVD_REPRESENTING_EMAIL_ADDRESS;
    spvProps[i++].Value.LPSZ    = m_pProps[EMAIL_ADDRESS].Value.LPSZ;

    SYSTEMTIME st;
    FILETIME ftTime;
    GetSystemTime (&st);
    SystemTimeToFileTime (&st, &ftTime);
    spvProps[i].ulPropTag = PR_MESSAGE_DOWNLOAD_TIME;
    spvProps[i++].Value.ft = ftTime;

    LPMAPITABLE pTable;
    LPSRowSet pRows;
    ULONG ulRows;
    BOOL fFoundCC, fFoundTO;
    fFoundCC = fFoundTO = FALSE;
    hResult = pMsgObj->GetRecipientTable (fMapiUnicode, &pTable);
    if (!hResult)
    {
        SPropValue spvFilter = { 0 };
        spvFilter.ulPropTag = PR_ADDRTYPE;
        spvFilter.Value.LPSZ = m_pProps[ADDRESS_TYPE].Value.LPSZ;

        SRestriction srRecips = { 0 };
        srRecips.rt = RES_PROPERTY;
        srRecips.res.resProperty.relop = RELOP_EQ;
        srRecips.res.resProperty.ulPropTag = PR_ADDRTYPE;
        srRecips.res.resProperty.lpProp = &spvFilter;

        hResult = pTable->Restrict (&srRecips, 0);
        if (!hResult)
        {
            hResult = pTable->GetRowCount (0, &ulRows);
            if (!hResult && ulRows)
            {
                hResult = pTable->SetColumns ((LPSPropTagArray)&sptMsgRecipProps, 0);
                if (!hResult)
                {
                    hResult = pTable->QueryRows (ulRows, 0, &pRows);
                    if (!hResult)
                    {
                        LPSPropValue pRecip;
                        ASSERT (pRows->cRows > 0);
                        for (ULONG ulIndex=0; ulIndex<pRows->cRows; ulIndex++)
                        {
                            pRecip = pRows->aRow[ulIndex].lpProps;
                            ASSERT (PR_EMAIL_ADDRESS == pRecip[0].ulPropTag);
                            ASSERT (PR_RECIPIENT_TYPE == pRecip[1].ulPropTag);
                            if (PR_EMAIL_ADDRESS == pRecip[0].ulPropTag)
                            {
                                // We use the address for the comparison because the sender might
                                // have create a one-off to send us this message and the only thing that this
                                // transport can count on being the same across all computers
                                // is the email address. The display name is modifyable by the user.
                                if (0 == lstrcmpi (pRecip[0].Value.LPSZ, m_pProps[EMAIL_ADDRESS].Value.LPSZ))
                                {
                                    if (PR_RECIPIENT_TYPE == pRecip[1].ulPropTag)
                                    {
                                        if (MAPI_TO == pRecip[1].Value.l)
                                        {
                                            fFoundTO = TRUE;
                                        }
                                        else
                                        {
                                            if (MAPI_CC == pRecip[1].Value.l)
                                            {
                                                fFoundCC = TRUE;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        FreeProws (pRows);
                    }
                }
            }
        }
        pTable->Release();
    }
    TraceResult ("CXPLogon::SetIncomingProps: Something went wrong setting PR_MESSAGE_xxx", hResult);

    if (!hResult)
    {
        spvProps[i].ulPropTag = PR_MESSAGE_TO_ME;
        spvProps[i++].Value.b = fFoundTO;

        spvProps[i].ulPropTag = PR_MESSAGE_CC_ME;
        spvProps[i++].Value.b = fFoundCC;

        spvProps[i].ulPropTag = PR_MESSAGE_RECIP_ME;
        spvProps[i++].Value.b = fFoundTO || fFoundCC;
    }
    hResult = pMsgObj->SetProps (i, spvProps, NULL);
    TraceResult ("CXPLogon::SetIncomingProps: Failed to set properties in the message", hResult);
    gpfnFreeBuffer (pProps);
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::GrowAdrList()
//
//    Parameters
//      ppAdrList           Pointer to an address where the old address list
//                          is and where the new resized address list will
//                          be returned
//      ulResizeBy          Number of new address entries to add to the list
//      ulOldAndNewCount    Number of entries in the old address list. In
//                          this parameter, upon sucessful return, will have
//                          the number of in the new address list
//
//    Purpose
//      In this function, given an address list with ulOldAndNewCount of
//      entries, we resize the address list to hold the old number of
//      entries plus the ulResizeBy entries. The old address list contents
//      are copied to the new list and the count reset. The memory for the
//      old address list is released here.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CXPLogon::GrowAdrList (LPADRLIST *   ppAdrList,
                                    ULONG         ulResizeBy,
                                    ULONG &       ulOldAndNewCount)
{
    LPADRLIST pNewAdrList;
    // Calculate how big the new buffer for the expanded address list should be
    ULONG cbSize = CbNewADRLIST (ulOldAndNewCount + ulResizeBy);
    // Allocate the memory for it
    HRESULT hResult = gpfnAllocateBuffer (cbSize, (LPVOID *)&pNewAdrList);
    if (hResult)
    {
        // We can't continue
        TraceResult ("CXPLogon::GrowAdrList: Failed to allocate memory for resized address list", hResult);
        return hResult;
    }

    // Zero-out all memory for neatness
    ZeroMemory (pNewAdrList, cbSize);

    // If we had entries in the old address list, copy the memory from
    // the old addres list into the new expanded list
    if (ulOldAndNewCount)
    {
        CopyMemory (pNewAdrList, *ppAdrList, CbNewADRLIST (ulOldAndNewCount));
    }

    // Set the number of entries in the new address list to the OLD size
    pNewAdrList->cEntries = ulOldAndNewCount;
    
    // We must return the number of available entries in the new expanded address list
    ulOldAndNewCount += ulResizeBy;

    // Free the old memory and put the new pointer in place
    gpfnFreeBuffer (*ppAdrList);
    *ppAdrList = pNewAdrList;
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::IsThereAMessageWaiting()
//
//    Parameters
//      cMsgCount   Reference to a variable that will return the number of 
//                  messages waiting to be processed
//
//    Purpose
//      Returns a the number of messages waiting in the directory.
//      It checks for a file name MSG_FILE_NAME in the same directory where 
//      this XP provider is stored. If it finds it, it sets the message counter
//      and returns.
//
//    Return Value
//      None
//
void WINAPI CXPLogon::IsThereAMessageWaiting (ULONG & cMsgCount)
{
    // Initialize the counter
    cMsgCount = 0;

    if (m_fPendingProcess)
    {
        return;
    }

    WIN32_FIND_DATA wfdFile = { 0 };
    BOOL fFound = FALSE;
    if (INVALID_HANDLE_VALUE != m_hSearch)
    {
        fFound = FindNextFile (m_hSearch, &wfdFile);
        if (!fFound)
        {
            FindClose (m_hSearch);
            m_hSearch = INVALID_HANDLE_VALUE;
        }
    }
    if (INVALID_HANDLE_VALUE == m_hSearch)
    {
        TCHAR szSearchMask[1024], szDirectory[MAX_PATH];
        if (!GetCurrentDirectory (MAX_PATH, szDirectory))
        {
            TraceMessage ("CXPLogon::IsThereAMessageWaiting: Failed to get current directory");
            return;
        }
        wsprintf (szSearchMask, TEXT("%s\\%s*.%s"), szDirectory, MSG_FILENAME_PREFIX, MSG_FILENAME_SUFFIX);
        m_hSearch = FindFirstFile (szSearchMask, &wfdFile);
        if (m_hSearch == INVALID_HANDLE_VALUE)
        {
            return;
        }
        fFound = TRUE;
    }
    else
    {
        fFound = FindNextFile (m_hSearch, &wfdFile);
    }
    if (fFound)
    {
        // We found a message
        cMsgCount = 1;
        m_fPendingProcess = TRUE;
        lstrcpy (m_szFileFound, wfdFile.cFileName);
    }
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::GetMsgFileName()
//
//    Parameters
//      pszFileName     Pointer where we return the name for a new message file
//      dwFileID        Where we return the ID of the file so the the
//                      attachment encoding function can use the same number.
//
//    Purpose
//      Create a unique name for a message about to be submitted using the
//      prefix and suffix for message file that this transport has defined. We
//      attempt to create a uniqure until not collision is found or a system
//      error occurs.
//
//    Return Value
//      TRUE if successful, FALSE otherwise.
//
BOOL WINAPI CXPLogon::GetMsgFileName (LPTSTR pszFileName, DWORD & dwFileID)
{
    m_dwNextID++;
    DWORD dwLastError = 0;
    HANDLE hFile;
    while (!dwLastError)
    {
        wsprintf (pszFileName, MSG_FILENAME_FORMAT, MSG_FILENAME_PREFIX, m_dwNextID, MSG_FILENAME_SUFFIX);
        hFile = CreateFile (pszFileName,
                            0,
                            0,
                            NULL,
                            CREATE_NEW,
                            FILE_ATTRIBUTE_ARCHIVE | FILE_FLAG_DELETE_ON_CLOSE,
                            NULL);
        if (INVALID_HANDLE_VALUE == hFile)
        {
            dwLastError = GetLastError ();
            if (ERROR_ALREADY_EXISTS)
            {   
                m_dwNextID++;
                dwLastError = 0;
                continue; // The WHILE() loop
            }
            TraceSysError ("CXPLogon::GetMsgFileName: Failed to create a file", dwLastError);
        }
        CloseHandle (hFile);
        dwFileID = m_dwNextID;
        break; // Out of the WHILE() loop
    }
    return dwLastError ? FALSE : TRUE;
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::CheckSpoolerYield()
//
//    Parameters                        
//      fReset      Resets the pseudo-timer
//
//    Purpose
//      Enforce the 0.2 second rule for transport that need to yield to the
//      MAPI spooler.  Called periodically while processing a message to
//      determine if we have used more than 0.2 seconds.  If so, then call
//      SpoolerYield(), else just continue.
//      This is called with fReset set to TRUE when we first enter one
//      of the Transport Logon methods (usually one that is known to
//      take a long time like StartMessage() or SubmitMessage(). )
//
//    Return Value
//      None.
//      
void WINAPI CXPLogon::CheckSpoolerYield (BOOL fReset)
{
    DWORD dwStop;
    static DWORD dwStart;
    if (fReset)
    {
        dwStart = GetTickCount();
    }
    else
    {
        dwStop = GetTickCount();
        if ((dwStop - dwStart) > 200) // 200 milliseconds
        {
            m_pSupObj->SpoolerYield (0);
            dwStart = GetTickCount();
        }
    }
}

// End of file for XPLOGON.CPP
