///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      XPWIZARD.CPP
//
//  Description
//      This file implements some of the entry point for the provider Profile
//      Wizard installation.
//
//  Author
//      Irving De la Cruz
//
//  Revision: 1.0
//
// Written for Microsoft Windows Developer Support
// Copyright (c) 1994-1995 Microsoft Corporation. All rights reserved.
//
#include "XPCUST.H"
#include <WINDOWSX.H>
#include <MAPIWZ.H>

#define DO_INFO_TRACES
#ifdef DO_INFO_TRACES
#define InfoTrace(a)   TraceInfoMessage(a)
#else
#define InfoTrace(a)   
#endif // DO_INFO_TRACES

// Prototypes of the function in this module
extern "C"
{
    WIZARDENTRY SampleWizardEntry;
    SERVICEWIZARDDLGPROC WizardDlgProc;
};

void WINAPI TogglePage (HWND, UINT, BOOL);

// Global data used in the WizardDlgProc function
LPMAPIPROP pPropObj = NULL;
SPropValue spvProps[NUM_LOGON_PROPERTIES];
BOOL fInitalizeControls = TRUE;
TCHAR szDisplayName[32], szEmailAddress[32], szAddressType[32];
UINT iPageNum = 0;

///////////////////////////////////////////////////////////////////////////////
//    SampleWizardEntry()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Returns the name of the resource where the controls for the wizard
//      pages are defined and outlined. The Wizard gives us a pointer to an
//      IMAPIProp object. We will release this object after we are finished
//      in the dialog procedure of our configuration pages.
//
//    Return Value
//      S_OK    Always
//
ULONG STDAPICALLTYPE SampleWizardEntry (HINSTANCE      hProviderDLLInstance,
                                        LPTSTR *       ppcsResourceName,
                                        DLGPROC *      ppDlgProc,
                                        LPMAPIPROP     pMapiProp,
                                        LPVOID         pMapiSupportObject)
{   
    InfoTrace ("SampleWizardEntry function called");
    // Quick sanity check on the input parameters
    ASSERT (ppcsResourceName);
    ASSERT (pMapiProp);
    ASSERT (ppDlgProc);
    // The resource name needs to be an actual text string not
    // something generated with the MAKEINTRESOURCE() macro
    const static TCHAR szTemplate[] = TEXT("WIZARD_PAGES");
    *ppcsResourceName = (LPTSTR)szTemplate;
    *ppDlgProc = (DLGPROC)WizardDlgProc;
    pPropObj = pMapiProp;
    pPropObj->AddRef();
    return S_OK;
}

///////////////////////////////////////////////////////////////////////////////
//    WizardDlgProc()
//
//    Parameters
//      { Refer to MAPI and Win32 SDK Documentation on this method }
//
//    Purpose
//      This is the function that the profile wizard will call while
//      interacting with our configuration pages.
//
//    Return Value
//      TRUE if we handle a message, FALSE if we don't or there was an error
//      in the data
//
BOOL STDAPICALLTYPE WizardDlgProc (HWND           hDlg,
                                   UINT           message,
                                   WPARAM         wParam,
                                   LPARAM         lParam)
{
    switch (message)
    {
        case WM_INITDIALOG:
            // If the control have not been initialized do so
    		if (fInitalizeControls)
    		{
                // Set the flag to avoid re-initialization
    			fInitalizeControls = FALSE;
                // Limit the maximum number of character users can
                // put in the edit fields
    			Edit_LimitText  (GetDlgItem (hDlg, IDC_DISPLAYNAME), sizeof(szDisplayName));
                Edit_LimitText  (GetDlgItem (hDlg, IDC_EMAILADDRESS), sizeof(szEmailAddress));
                Edit_LimitText  (GetDlgItem (hDlg, IDC_ADDRESSTYPE), sizeof(szAddressType));
                SetWindowText (GetDlgItem (hDlg, IDC_ADDRESSTYPE), TEXT("XPCUST"));
                // Zero out the contents of property array
                ZeroMemory (spvProps, sizeof(SPropValue) * NUM_LOGON_PROPERTIES);
    		}
    		return TRUE;

        case WIZ_QUERYNUMPAGES :
            // Return the number of pages the our provider needs to display
            return (BOOL)3;

        case WM_CLOSE :
            pPropObj->Release();
            pPropObj = NULL;
		    return TRUE;

        case WM_COMMAND :
		    switch (LOWORD(wParam))
		    {
		        case WIZ_NEXT:
		            switch (iPageNum)
			        {
                        // We are about to leave a page in each of the 
                        // following cases. Here we should get the data from
                        // the controls in that page, validate it if necessary
                        // and copy the validated data into the property array
			            case 0:
                            break;

			            case 1:
                            if (!GetWindowText (GetDlgItem (hDlg, IDC_DISPLAYNAME),
    				                            szDisplayName,
    				                            sizeof(szDisplayName)))
            				{
            					MessageBeep (MB_ICONEXCLAMATION);
            					return FALSE;
            				}
            				spvProps[DISPLAY_NAME].ulPropTag  = PR_SKEL_USER_NAME;
                            spvProps[DISPLAY_NAME].Value.LPSZ = szDisplayName;
            				break;

			            case 2:
            				if (!GetWindowText (GetDlgItem (hDlg, IDC_EMAILADDRESS),
                                                szEmailAddress,
                                                sizeof(szEmailAddress)))
            				{
            					MessageBeep (MB_ICONEXCLAMATION);
            					return FALSE;
            				}
            				spvProps[EMAIL_ADDRESS].ulPropTag  = PR_EMAIL_ADDRESS;
                            spvProps[EMAIL_ADDRESS].Value.LPSZ = szEmailAddress;
            				break;

        			    case 3:
                            if (!GetWindowText (GetDlgItem (hDlg, IDC_ADDRESSTYPE),
                                                szAddressType,
                                                sizeof(szAddressType)))
            				{
            					MessageBeep (MB_ICONEXCLAMATION);
            					return FALSE;
            				}
            				spvProps[ADDRESS_TYPE].ulPropTag  = PR_ADDRTYPE;
                            spvProps[ADDRESS_TYPE].Value.LPSZ = szAddressType;

                            // Since this is the last page in our configuration
                            // we must save the obtained data into the
                            // IMAPIProp object that the wizard gave us. In
                            // turn this data will be saved in the profile
                            // section of this provider
                            ASSERT (pPropObj);
                            if (pPropObj)
                            {
				                HRESULT hResult = pPropObj->SetProps (NUM_LOGON_PROPERTIES,
				                                                      spvProps,
				                                                      NULL);
                                if (hResult)
                                {
                                    // Something went wrong
                                    TraceResult ("WizardDlgPrc: Failed to set props", hResult);
                                    return FALSE;
                                }
                            }
            				break;

        			    default:
                            // We should not get here. No pages after three
        				    ASSERT (FALSE);
        				    break;
			        }
                    // Disable the page we were in and enable the next one
                    // before it is display by the wizard
                    TogglePage (hDlg, iPageNum, FALSE);
                    TogglePage (hDlg, ++iPageNum, TRUE);
        			return TRUE;

        		case WIZ_PREV:
                    // Disable the page we were in and enable the next one
                    // before it is display by the wizard
        			TogglePage (hDlg, iPageNum, FALSE);
        			TogglePage (hDlg, --iPageNum, TRUE);
        			return TRUE;
		    }
		    break;
    }
    return FALSE;
}

///////////////////////////////////////////////////////////////////////////////
//    TogglePage()
//
//    Parameters
//      hDlg        Handle to the dialogs where the controls are located
//      iPage       Number of the page we are going to toggle
//      fState      State of the controls: TRUE shows them, FALSE hides them
//
//    Purpose
//      This function will hide or show the controls for the page number
//      specified in the iPage parameter.
//
//    Return Value
//      None
//
void WINAPI TogglePage (HWND hDlg, UINT iPage, BOOL fState)
{
    HWND hEditCtl = NULL;
    // What page is it?
    switch (iPage)
    {
        // Toggle the controls that should be displayed for a particular page
        case 1 :
            hEditCtl = GetDlgItem (hDlg, IDC_DISPLAYNAME);
            EnableWindow (GetDlgItem (hDlg, IDC_STATIC1), fState);
            ShowWindow   (GetDlgItem (hDlg, IDC_STATIC1), fState ? SW_SHOW : SW_HIDE);
            EnableWindow (GetDlgItem (hDlg, IDC_INFO1), fState);
            ShowWindow   (GetDlgItem (hDlg, IDC_INFO1), fState ? SW_SHOW : SW_HIDE);
            break;

        case 2 :
            hEditCtl = GetDlgItem (hDlg, IDC_EMAILADDRESS);
            EnableWindow (GetDlgItem (hDlg, IDC_STATIC2), fState);
            ShowWindow   (GetDlgItem (hDlg, IDC_STATIC2), fState ? SW_SHOW : SW_HIDE);
            EnableWindow (GetDlgItem (hDlg, IDC_INFO2), fState);
            ShowWindow   (GetDlgItem (hDlg, IDC_INFO2), fState ? SW_SHOW : SW_HIDE);
            break;

        case 3 :
            hEditCtl = GetDlgItem (hDlg, IDC_ADDRESSTYPE);
            EnableWindow (GetDlgItem (hDlg, IDC_STATIC3), fState);
            ShowWindow   (GetDlgItem (hDlg, IDC_STATIC3), fState ? SW_SHOW : SW_HIDE);
            EnableWindow (GetDlgItem (hDlg, IDC_INFO3), fState);
            ShowWindow   (GetDlgItem (hDlg, IDC_INFO3), fState ? SW_SHOW : SW_HIDE);
            break;
    }
    if (hEditCtl)
    {
        EnableWindow (hEditCtl, fState);
        ShowWindow   (hEditCtl, fState ? SW_SHOW : SW_HIDE);
        if (fState)
        {
            SetFocus (hEditCtl);
        }
    }
}

// End of file for XPWIZARD.CPP
